<?php

namespace App\Model;

use Nette;
use Nette\Database\Table\ActiveRow;

/**
 * Class for handling activities related to teams
 * @package App\Model
 */
class TeamManager {
    use Nette\SmartObject;

    private Nette\Database\Explorer $database;
    private StudentManager $studentManager;

    public function __construct(Nette\Database\Explorer $database,
                                StudentManager $studentManager) {
        $this->database = $database;
        $this->studentManager = $studentManager;
    }

    /**
     * Returns array [index of skill, name of skill] by team
     * @param int $teamId
     * @return array
     */
    public function getTeamAdvances(int $teamId) {

        return $this->database->query('select tags_id,name from TEAM_SKILL,TAG where tags_id = id and teams_id = ?;', $teamId)->fetchPairs("tags_id", "name");
    }

    /**
     * Function returns project related to given team record
     * @param $team team record
     * @return mixed
     */
    public function getProjectRelToTeam($team) {
        return $team->related("PROJECT")->fetch();
    }

    /**
     * Function returns leader user of given team
     * @param int $teamId team identifier
     * @return Nette\Database\Table\ActiveRow|null
     */
    public function getTeamLeader(int $teamId) {
        $team = $this->getTeam($teamId);
        $teamLeader = $this->database->table('STUDENT')->get((int)$team["student_id"]);
        return $teamLeader ?? null;
    }

    /**
     * Function returns array of team members
     * @param int $teamId team identifier
     * @return Nette\Database\Table\ActiveRow[]
     */
    public function getTeamMembers(int $teamId) {
        $teamLeaderId = $this->getTeamLeader($teamId)->id;
        $teamMembers = $this->database->table('TEAM')->get($teamId)->related("STUDENT")->order("lastname ASC")->fetchAll();

        $result = [$teamLeaderId=>$teamMembers[$teamLeaderId]]+$teamMembers;
        return $result;
    }

    /**
     * Function returns student list of given team without leader
     * @param int $leaderId leader student identifier
     * @param int $teamId team identifier
     * @return Nette\Database\Table\ActiveRow[]
     */
    public function getTeamMembersExceptLeader(int $leaderId, int $teamId) {
        return $this->database->table('TEAM')->get($teamId)->related("STUDENT")->where("id != ?",$leaderId)->select('id, CONCAT(firstname," ",lastname," (",personalID,")") AS studentInfo')->order('lastname ASC')->fetchPairs("id",'studentInfo');
    }

    /**
     * Function returns team record in array
     * @param int $teamId team identifier
     * @return array
     */
    public function getTeam(int $teamId){
        return $this->database->table("TEAM")->get($teamId)->toArray();
    }

    /**
     * Function returns team record
     * @param int $teamId team identifier
     * @return ActiveRow|null
     */
    public function getTeamById(int $teamId) {
        return $this->database->table("TEAM")->get($teamId);
    }

    /**
     * Function to update team information from given parameters
     * @param int $teamId team identifier
     * @param string $teamName team name
     * @param array $teamMembers team members
     */
    public function updateTeam(int $teamId, string $teamName, array $teamMembers) {
        $team = $this->database->table("TEAM")->get($teamId);
        $team->update(["name"=> $teamName]);
        $this->database->table("STUDENT")->where("team_id = ?",$teamId)->update(["team_id"=> null]);
        $this->database->table("STUDENT")->where("id = ?",$team["student_id"])->update(["team_id"=> $teamId]);

        //bdump($this->database->table('TEAM_ROLE_STUDENT')->where('student.team_id IS NULL')->delete());

        foreach ($teamMembers as $member) {
            $this->database->table("STUDENT")->where("id",$member)->update(["team_id" => $teamId]);
        }

        $to_delete = $this->database->table('TEAM_ROLE_STUDENT')->where('STUDENT.team_id IS NULL')->fetchAll();

        foreach ($to_delete as $member) {
            $member->delete();
        }

        //$this->database->table('TEAM_ROLE_STUDENT')->where('STUDENT.team_id IS NULL')->delete();
    }

    /**
     * Function to change team state
     * @param bool $state Marks team as complete - true, or incomplete - false
     */
    public function changeTeamState(int $teamId,bool $state) {
        $team = $this->database->table("TEAM")->get($teamId);
        $team->update(["is_complete"=> $state]);
    }

    /**
     * Function to get students's team identifier
     * @param int $studentId student identifier
     * @return mixed
     */
    public function getTeamIdRelByStudent(int $studentId) {
        return $this->database->table("STUDENT")->get($studentId)["team_id"];
    }

    /**
     * Updating team skills
     * @param int $teamId team Id
     * @param array $skillIds array of skills, that comes from multiselect
     */
    public function updateTeamSkills(int $teamId, array $skillIds) {

        $this->database->table("TEAM_SKILL")->where("teams_id", $teamId)->delete();

        $values = Utils::getArrayForDynamicInsert($teamId,$skillIds,"teams_id","tags_id");
        if(count($values) > 0) {
            $this->database->table("TEAM_SKILL")->insert($values);
        }
    }

    /**
     * Function to check if team has assigned assignment
     * @param int $teamId team identifier
     * @return bool
     */
    public function hasTeamAssignment(int $teamId):bool {
        return $this->database->table('TEAM')->get($teamId)->related('PROJECT')->count() > 0;
    }

    /**
     * Function to check if team has assigned assignment
     * @param ActiveRow $team team identifier
     * @return bool
     */
    public function hasTeamAssignmentAR(Nette\Database\Table\ActiveRow $team) {
        return $team->related('PROJECT')->count() > 0;
    }

    /**
     * Function returns selection of teams marked as incomplete
     * @return Nette\Database\Table\Selection
     */
    public function getIncompleteTeams() {
        return $this->database->table('TEAM')->where('is_complete', 0);
    }

    /**
     * Function to get team role information
     * @param ActiveRow $team team record
     * @return Nette\Database\Table\Selection
     */
    public function getTeamRoleInfo(ActiveRow $team) {

        $roles = $this->database->table('TEAM_ROLE_STUDENT')->where('STUDENT.team_id', $team->id)
            ->group('TEAM_ROLE.label')->select('TEAM_ROLE.label')->select('TEAM_ROLE.color')
            ->select('COUNT(TEAM_ROLE.label) AS num')->order('num DESC');

        return $roles;
    }

    /**
     * Function to update credit date record information
     * @param $teamId team identifier
     * @param $creditTsp1 date information of TSP1
     * @param $creditTsp2 date information of TSP2
     */
    public function updateCredit($teamId, $creditTsp1, $creditTsp2) {
        $this->getTeamById($teamId)->related('PROJECT')->fetch()->update([
            'course_credit_tsp1' => $creditTsp1 == '' ? null : $creditTsp1,
            'course_credit_tsp2' => $creditTsp2 == '' ? null : $creditTsp2,
        ]);
    }

    /**
     * Function to create new team based on team leader
     * @param $leaderId team leader user identifier
     * @param $name initial team name
     * @return ActiveRow|null
     */
    public function createTeam($leaderId, $name):ActiveRow|null {
        $leader = $this->studentManager->getStudent($leaderId);

        $team = $this->database->table('TEAM')->insert([
            "period" => $leader->period,
            "student_id" => $leader->id,
            "name" => $name,
        ]);

        $leader->update([
            'team_id' => $team->id,
        ]);

        return $team;

    }
}