<?php

namespace App\Model;

use Nette;

/**
 * Class for handling activities related to team roles
 * @package App\Model
 */
class TeamRoleManager {
    use Nette\SmartObject;

    private Nette\Database\Explorer $database;

    /**
     * @var TeamManager
     */
    private TeamManager $teamManager;

    public function __construct(Nette\Database\Explorer $database, TeamManager $teamManager) {
        $this->database = $database;
        $this->teamManager = $teamManager;
    }

    /**
     * Function to get team role record from given identifier
     * @param $roleId team role identifier
     * @return Nette\Database\Table\ActiveRow|null
     */
    public function getRole($roleId) {
        return $this->database->table('TEAM_ROLE')->get($roleId);
    }

    /**
     * Function returns list of all roles
     * @return Nette\Database\Table\Selection
     */
    public function getAllRoles() {

        return $this->database->table("TEAM_ROLE");
    }

    /**
     * Function returns list of all roles in pair format
     * @return array
     */
    public function getAllRolesForSelect() {
        return $this->getAllRoles()->fetchPairs("id", "label");
    }

    /**
     * Function returns list of all applicable roles in pair format
     * @return array
     */
    public function getAllApplicableRolesForSelect() {
        return $this->getAllRoles()->where('applicable', 1)->order("label DESC")->fetchPairs("id","label");
    }


    /**
     * Function returns all roles of given team member
     * @param int $studentId student identifier
     * @return Nette\Database\ResultSet
     */
    public function getAllRolesForStudent(int $studentId) {
        return $this->database->query(
            'select team_role_id,label,color 
            from TEAM_ROLE,TEAM_ROLE_STUDENT 
            where TEAM_ROLE_STUDENT.student_id = ? and TEAM_ROLE.id = TEAM_ROLE_STUDENT.team_role_id and TEAM_ROLE.applicable = 1'
            ,$studentId
        );
    }

    /**
     * Function to get all team member's roles of given team
     * @param int $teamId team identifier
     * @return array
     */
    public function getAllRolesForTeam(int $teamId){
        $team = $this->teamManager->getTeamMembers($teamId);
        $roles = [];
        foreach ($team as $member){
            $roles[$member["id"]] = $this->getAllRolesForStudent($member["id"])->fetchPairs("label","color");
        }

        return $roles;
    }

    /**
     * Function to get student's roles in pairs array
     * @param int $studentId
     * @return array
     */
    public function getAllRolesForStudentForSelect(int $studentId){
        return $this->getAllRolesForStudent($studentId)->fetchPairs("team_role_id","label");
    }

    /**
     * Function to update team roles of given students
     * @param int $studentId student identifier
     * @param array $roleIds new array of students team roles
     */
    public function updateStudentRoles(int $studentId,array $roleIds) {
        $this->database->table("TEAM_ROLE_STUDENT")->where("student_id", $studentId)->delete();
        foreach ($roleIds as $roleId) {
            $this->database->table("TEAM_ROLE_STUDENT")->insert(["student_id" => $studentId, "team_role_id" => $roleId]);
        }
    }

    /**
     * Function to create new team role
     * @param string $label team role label
     * @param int $applicable flag if is flag applicable (1 = yes; 0 = no)
     * @param string $color team role badge color
     * @return bool
     */
    public function createRole(string $label, int $applicable, string $color) {

        $row = $this->database->table('TEAM_ROLE')->insert([
            'label' => $label,
            'applicable' => $applicable,
            'color' => $color,
        ]);

        if($row) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Function to edit existing team role information
     * @param int $roleId team role identifier
     * @param string $label team role label
     * @param int $applicable flag if is flag applicable (1 = yes; 0 = no)
     * @param string $color team role badge color
     * @return bool
     */
    public function editRole(int $roleId, string $label, int $applicable, string $color) {

        $row = $this->getRole($roleId)->update([
            'label' => $label,
            'applicable' => $applicable,
            'color' => $color,
        ]);

        if($row) {
            return true;
        } else {
            return false;
        }
    }
}
