<?php

namespace App\Presenters;

use App\Model\AssignmentManager;
use App\Model\Constants;
use App\Model\InterestManager;
use App\Model\RoleManager;
use App\Model\StudentManager;
use App\Model\TeamManager;
use App\Model\UserManager;
use App\Model\Utils;
use Nette\Application\UI\Form;
use Nette;

/**
 * Presenter class to handle requests to page with assignment details
 */
class AssignmentPresenter extends BasePresenter {

    public AssignmentManager $assignmentManager;
    public StudentManager $studentManager;
    public InterestManager $interestManager;
    public TeamManager $teamManager;

    public function __construct(AssignmentManager $assignmentManager,
    StudentManager $studentManager, InterestManager $interestManager, TeamManager $teamManager) {
        parent::__construct();
        $this->assignmentManager = $assignmentManager;
        $this->studentManager = $studentManager;
        $this->interestManager = $interestManager;
        $this->teamManager = $teamManager;
    }

    public function renderShow(int $id):void {

        $assignment = $this->assignmentManager->getAssignment($id);

        $this->template->tags = $assignment->related('ASSIGNMENT_TAG');

        $this->template->assignment = $assignment;

        $this->template->creator = $assignment->contact;

        $this->template->mentor = $assignment->user;

        $this->template->isAssigned = $this->assignmentManager->isAssigned($id);
        if($this->template->isAssigned) {
            $this->template->assignedProject = $this->assignmentManager->getAssignedProject($id);
        }

        $this->template->teamHasProject = false;
        $this->template->inInterest = false;

        $this->template->assignmentFiles = $this->assignmentManager->getAssignmentFiles($assignment);

        if($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {
            $this->template->team = $this->studentManager->teamRelToStudent($this->studentManager->getStudentByUser($this->getUser()->getId()));
            $this->template->interests = $this->interestManager->getActiveInterests($this->template->team);

            $student = $this->studentManager->getStudentByUser($this->getUser()->getId());
            $this->template->teamHasProject = $this->teamManager->hasTeamAssignment($this->teamManager->getTeamIdRelByStudent($student->id));
            $this->template->inInterest = $this->interestManager->hasTeamInterest($this->studentManager->teamRelToStudent($student), $id);
        }
    }

    public function renderDefault():void {

        $project = $this->studentManager->projectRelToStudent($this->studentManager
            ->getStudent($this->getUser()->getIdentity()->contact_id))->fetch();
        if(isset($project)){
            $this->redirect("Assignment:show",$project['assignment_id']);
        }

    }

    public function handleDownloadAssignmentFile(int $fileId) {

        $file = $this->assignmentManager->getAssignmentFile($fileId);

        if($file == null) {
            $this->flashMessage('Chyba;Soubor se nepodařilo stáhnout.', 'bg-danger');
            return;
        }

        header('Content-length: ' . $file->filesize);
        header('Content-type: application/pdf');
        header('Content-disposition: attachment; filename=' . $file->filename);
        echo $file->file;
        if(ob_get_length() > 0) {
            ob_clean();
        }
        flush();

        $assignment = $file->related('ASSIGNMENT')->fetch();

        if($this->getUser()->isLoggedIn()) {
            $this->loggerManager->get('default')->info('The user downloaded a file with the details of the assignment', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'assignment_file_id' => $file->id,
                'assignment_id' => $assignment->id,
                'assignment_name' => $assignment->name,
            ]);
        } else {
            $this->loggerManager->get('default')->info('The visitor downloaded a file with the details of the assignment', [
                'ip_addr' => UserManager::getIp(),
                'assignment_file_id' => $file->id,
                'assignment_id' => $assignment->id,
                'assignment_name' => $assignment->name,
            ]);
        }

        exit;
    }

    public function handleLeaderAssignedAssignment(int $assignmentId) {

        $interest = $this->interestManager->createInterest($assignmentId,
            $this->studentManager->teamRelToStudent($this->studentManager->getStudentByUser($this->getUser()->getId())),
            new Nette\Utils\DateTime());
        $this->loggerManager->get('default')->notice('The team leader expressed interest in the assignment', [
            'user_identity' => $this->getUser()->getIdentity()->getData(),
            'interest_id' => $interest->id,
            'interest_weight' => $interest->weight,
            'team_id' => $interest->team->id,
            'team_name' => $interest->team->name,
            'assignment_id' => $interest->assignment->id,
            'assignment_name' => $interest->assignment->name,
        ]);
        $this->flashMessage("Úspěch;Váš tým má nyní vybrané téma.", "bg-success");
        $this->redirect('this');
    }

    public function handleMentorMentorAssignment() {

        if(!$this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
            $this->flashMessage("Chyba;Na provedení této akce nemáte potřebná oprávnění.", "bg-danger");
            return;
        }

        $assignment_id = $this->getParameter('id');
        $assignment = $this->assignmentManager->getAssignment($assignment_id);
        if(!$this->assignmentManager->assignMentor($assignment, $this->getUser()->getId())) {
            $this->flashMessage("Chyba;Při validaci požadavku se vyskytla chyba.", "bg-danger");
            return;
        }

        $this->loggerManager->get('default')->notice('The mentor assigned an assignment', [
            'user_identity', $this->getUser()->getIdentity()->getData(),
            'assignment_id', $assignment->id,
            'assignment_name', $assignment->name,
        ]);
        $this->flashMessage("Úspěch;Toto zadání nyní mentorujete.", "bg-success");
        $this->redirect('this');
    }

    protected function createComponentShowInterestForm(): Form {
        $form = new Form;

        $optionStrings = array(
            Constants::$ASSIGNMENT_INTEREST_WEIGHT_VERY_LOW     => "Velmi nízký",
            Constants::$ASSIGNMENT_INTEREST_WEIGHT_LOW          => "Nízký",
            Constants::$ASSIGNMENT_INTEREST_WEIGHT_NORMAL       => "Střední",
            Constants::$ASSIGNMENT_INTEREST_WEIGHT_SERIOUS      => "Velký",
            Constants::$ASSIGNMENT_INTEREST_WEIGHT_VERY_SERIOUS => "Velmi velký"
        );

        $interests = $this->interestManager->getActiveInterests($this->studentManager->teamRelToStudent($this->studentManager->getStudentByUser($this->getUser()->getId())));

        foreach ($interests as $interest) {
            unset($optionStrings[$interest->weight]);
        }

        if (sizeof($optionStrings) > 0) {
            $form->addSelect('weight', "Síla zájmu", $optionStrings
                , 1)
                ->setHtmlAttribute('class', 'form-control');

            $form->addSubmit('send', "Projevit zájem")
                ->setHtmlAttribute("class", "form-control")
                ->setHtmlId("frm-showInterestForm-submit");
        } else {
            $form->addSelect('weight', "Síla zájmu", $optionStrings
                , 1)
                ->setHtmlAttribute('class', 'form-control')->setDisabled();

            $form->addSubmit('send', "Projevit zájem")
                ->setHtmlAttribute("class", "form-control")
                ->setHtmlId("frm-showInterestForm-submit")->setDisabled();
        }

        $form->onSuccess[] = [$this, 'showInterestFormSucceeded'];

        return $form;
    }

    public function showInterestFormSucceeded(array $values):void {
        if(!$this->getUser()->isLoggedIn()) {
            $this->redirect("Sign:in");
        }

        $assignmentId = $this->getParameter('id');
        $team = $this->studentManager->teamRelToStudent($this->studentManager->getStudentByUser($this->getUser()->getId()));

        $this->database->beginTransaction();
        if($this->interestManager->getActiveInterests($team)->count() < 5) {

            $interest = $this->interestManager->createInterest($assignmentId, $team,
                new Nette\Utils\DateTime(), $values['weight']);
            $this->database->commit();

            $this->loggerManager->get('default')->notice('The team leader expressed interest in the assignment', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'interest_id' => $interest->id,
                'interest_weight' => $interest->weight,
                'team_id' => $interest->team->id,
                'team_name' => $interest->team->name,
                'assignment_id' => $interest->assignment->id,
                'assignment_name' => $interest->assignment->name,
            ]);

            $this->flashMessage("Úspěch;Váš zájem o téma byl vyjádřen.", "bg-success");
        } else {
            $this->database->commit();
            $this->flashMessage('Chyba;Při projevení zájmu se vyskytla chyba.', "bg-warning");
        }

        $this->redirect('this');
    }

    public function canAccessPage(): bool {
        if($this->getAction() == 'default') {
            return true;
        }

        $id = $this->getParameter('id');

        if(!isset($id)) {
            return false;
        }

        $assignment = $this->assignmentManager->getAssignment($id);

        if($assignment) {

            if ($this->getUser()->isInRole(RoleManager::$ROLE_GARANT)) {
                return true;
            }

            if ($assignment->hidden == Constants::$ASSIGNMENT_VISIBILITY_PUBLIC) {
                return true;
            }

            if ($this->getUser()->isInRole(RoleManager::$ROLE_MENTOR)) {
                if ($assignment->hidden == Constants::$ASSIGNMENT_VISIBILITY_MENTOR_ONLY) {
                    return true;
                }

                if ($this->getUser()->getId() == $assignment->user->id) {
                    return true;
                } else {
                    return false;
                }

            }

            if ($this->getUser()->isInRole(RoleManager::$ROLE_LEADER)) {

                $student = $this->studentManager->getStudentByUser($this->getUser()->getId());
                $team = $this->studentManager->teamRelToStudent($student);
                $userProject = $this->teamManager->getProjectRelToTeam($team);

                if (!$userProject) {
                    return false;
                }

                if ($id == $userProject->assignment->id) {
                    return true;
                } else {
                    return false;
                }

            }
        }

        return parent::canAccessPage(); // TODO: Change the autogenerated stub

    }
}