<?php

declare(strict_types=1);

namespace App\Presenters;

use App\Components\DefenceAddForm;
use App\Components\TeamCreditForm;
use App\Components\TeamEditForm;
use App\Model\DefenceManager;
use App\Model\MeetingFacade;
use App\Model\ProjectManager;
use App\Model\RoleManager;
use App\Model\TeamManager;
use Nette;
use Nette\Application\UI\Form;
/**
 * Presenter class to handle requests to page with team management
 */
class TeamManagementPresenter extends BasePresenter {

    protected ProjectManager $projectManager;
    protected DefenceManager $defenceManager;
    protected MeetingFacade $meetingFacade;
    protected TeamEditForm $teamEditForm;
    protected TeamCreditForm $teamCreditForm;
    protected DefenceAddForm $defenceAddForm;

    public Nette\Database\Explorer $database;

    public function __construct(ProjectManager $projectManager,
                                MeetingFacade $meetingFacade,
                                TeamEditForm $teamEditForm,
                                TeamCreditForm $teamCreditForm,
                                DefenceManager $defenceManager,
                                DefenceAddForm $defenceAddForm) {
        parent::__construct();

        $this->projectManager = $projectManager;
        $this->meetingFacade = $meetingFacade;
        $this->teamEditForm = $teamEditForm;
        $this->teamCreditForm = $teamCreditForm;
        $this->defenceManager = $defenceManager;
        $this->defenceAddForm = $defenceAddForm;
    }

    public function startup() {
        parent::startup();

        if($this->getAction() == 'editTeam' && empty($this->getParameter('do'))) {

            $id = intval($this->getParameter('id'));

            $team = $this->teamManager->getTeamById($id);
            if(!$team) {
                $this->flashMessage('Chyba;Tým neexistuje', 'bg-danger');
                $this->redirect('TeamManagement:');
            }
        }
    }

    public function renderEditTeam(int $id) {
        $this->template->project = $this->teamManager->getProjectRelToTeam($this->teamManager->getTeamById($id));
        $this->template->defences = $this->defenceManager->getDefencesRelToProject($this->template->project);
    }

    public function renderDefault() {

        $this->template->projects = $this->projectManager->getProjectsWithInfo();

        $this->template->meetings = $this->meetingFacade->getMeetingsLinkedWith('mentor', $this->getUser()->getId());

        $this->loggerManager->get('default')->info("Mentored projects page is loaded",["user_id"=>$this->getUser()->getId()]);
    }

    public function createComponentTeamEditForm():Form {
        $teamId = intval($this->getParameter('id'));
        $form = $this->teamEditForm->create($teamId);

        $team = $this->teamManager->getTeamById($teamId);

        $form->onSuccess[] = function() use ($team) {
            $this->loggerManager->get('default')->info('Team information was modified', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'team_id' => $team->id,
                'team_name' => $team->name,
            ]);
            $this->flashMessage('Úspěch;Tým byl úspěšně upraven.', 'bg-success');
            $this->redirect('this');
        };

        return $form;
    }

    public function createComponentTeamCreditForm():Form {
        $teamId = intval($this->getParameter('id'));
        $form = $this->teamCreditForm->create($teamId);

        $team = $this->teamManager->getTeamById($teamId);

        $form->onSuccess[] = function() use ($team) {
            $this->loggerManager->get('default')->info('Team credits information was modified', [
            'user_identity' => $this->getUser()->getIdentity()->getData(),
            'team_id' => $team->id,
            'team_name' => $team->name,
        ]);
            $this->flashMessage('Úspěch;Tým byl úspěšně upraven.', 'bg-success');
            $this->redirect('this');
        };

        return $form;
    }

    public function createComponentDefenceAddForm():Form {
        $teamId = intval($this->getParameter('id'));
        $form = $this->defenceAddForm->create($teamId);

        $team = $this->teamManager->getTeamById($teamId);

        $form->onSuccess[] = function() use ($team) {
            $this->loggerManager->get('default')->info('Team defence record created', [
                'user_identity' => $this->getUser()->getIdentity()->getData(),
                'team_id' => $team->id,
                'team_name' => $team->name,
            ]);
            $this->flashMessage('Úspěch;Tým byl úspěšně upraven.', 'bg-success');
            $this->redirect('this');
        };

        return $form;
    }
    public function canAccessPage(): bool {
        if(!$this->user->isLoggedIn()) {
            return false;
        } else if($this->user->isInRole(RoleManager::$ROLE_GARANT)) {
            return true;
        }
        return false;
    }
}