<?php

namespace App\Components;

use App\Model\ProjectManager;
use App\Model\StudentManager;
use App\Model\TeamManager;
use App\Model\TagManager;
use App\Model\TeamRoleManager;
use App\Model\UserManager;
use App\Model\Utils;
use Nette\Application\AbortException;
use Nette\Application\UI\Form;
use Nette\Database\Explorer;
use Nette\Security\AuthenticationException;
use Nette\Security\User;
use Nette\Utils\ArrayHash;

/**
 * Form component class for edit project description links
 * @package App\Components
 */
class ProjectLinksForm extends Form {
    protected ProjectManager $projectManager;

    /**
     * Component constructor
     * @param ProjectManager $projectManager
     */
    public function __construct(ProjectManager $projectManager) {
        parent::__construct();
        $this->projectManager = $projectManager;
    }

    /**
     * Factory function for creating description edit form
     * @param int $projectId project identifier (<0 invalid identifier)
     * @return Form resulted form instance
     * @throws \Exception
     */
    public function create(int $projectId = -1) {
        $defaults = array();
        $form = new Form();
        $renderer = $form->getRenderer();
        $renderer->wrappers['controls']['container'] = 'div class="m-2"';
        $renderer->wrappers['pair']['container'] = 'div class="form-group"';


        if ($projectId < 0) {
            throw new \Exception("Cannot create form with invalid id");
        }

        $defaults = $this->projectManager->getProject($projectId);

        $form->addText("prj_storage", "Úložiště projektu")
            ->setHtmlAttribute("class", "form-control")
            ->setHtmlAttribute("placeholder", "Vložte odkaz")
            ->addRule($form::URL, "Adresa musí být odkaz!")
            ->setDefaultValue($defaults["prj_storage"]);

        $form->addText("project_management", "Řízení projektu")
            ->setHtmlAttribute("class", "form-control")
            ->setHtmlAttribute("placeholder", "Vložte odkaz")
            ->addRule($form::URL, "Adresa musí být odkaz!")
            ->setDefaultValue($defaults["project_management"]);

        $form->addText("app_production", "Produkční verze aplikace")
            ->setHtmlAttribute("class", "form-control")
            ->setHtmlAttribute("placeholder", "Vložte odkaz")
            ->addRule($form::URL, "Adresa musí být odkaz!")
            ->setDefaultValue($defaults["app_production"]);

        $form->addTextArea("other_links", "Další odkazy", 2, 4)
            ->setHtmlAttribute("class", "form-control")
            ->setHtmlAttribute("placeholder", "Vložte odkazy ve tvaru\nnazev;url\nnazev;url")
            ->setDefaultValue($defaults["other_links"]);


        $form->addHidden("projectId", $projectId);
        $form->addSubmit('save', 'Potvrdit změny')->setHtmlAttribute('class', 'btn btn-primary')
            ->setHtmlId("frm-projectLinksForm-submit");
        $form->addButton('cancel', 'Zrušit')
            ->setHtmlAttribute('class', 'btn btn-link')
            ->setHtmlId("frm-projectLinksForm-cancel");;

        $form->onValidate[] = [$this, 'validateForm'];
        $form->onSuccess[] = [$this, 'saveForm'];

        return $form;
    }

    /**
     * Function for validation form entries
     * @param Form $form
     * @param ArrayHash $values
     */
    public function validateForm(Form $form, ArrayHash $values) {

        $text = $values["other_links"];
        $result = Utils::validateOtherProjectLinksFormat($text);
        if($result==false) {
            $form->addError("Zadané odkazy nejsou ve formátu: NAZEV;URL.");
        }
    }

    /**
     * Function that is triggered by a successful form submission
     * @param Form $form
     * @param ArrayHash $values
     */
    public function saveForm(Form $form, ArrayHash $values) {

        try {
            $this->projectManager->getProject($values["projectId"])->update(
                ["prj_storage" => $values["prj_storage"],
                    "project_management" => $values["project_management"],
                    "app_production" => $values["app_production"],
                    "other_links" => $values["other_links"]
                ]
            );
        } catch (\Exception $e) {

        }
    }
}