<?php

namespace App\Components;

use App\Model\TeamManager;
use App\Model\TeamRoleManager;
use Nette\Application\UI\Form;
use Nette\Database\Explorer;
use Nette\Utils\ArrayHash;

/**
 * Form component class for creating team roles.
 */
class TeamRoleAddForm extends Form {

    protected Explorer $database;
    protected TeamRoleManager $teamRoleManager;

    /**
     * Component constructor
     * @param Explorer $database
     * @param TeamRoleManager $teamRoleManager
     */
    public function __construct(Explorer $database, TeamRoleManager $teamRoleManager) {
        parent::__construct();

        $this->teamRoleManager = $teamRoleManager;
        $this->database = $database;
    }

    /**
     * Factory function for creating (edit) team role creation form
     * @param int $roleId team role identification for edit purpose (=0 - creating new team role)
     * @return Form
     * @throws \Exception
     */
    public function create($roleId = 0):Form {
        $defaults = [
            'name' => '',
            'roleColor' => 'badge-primary',
            'canUse' => true,
        ];

        $form = new Form();

        if($roleId != 0) {
            $role = $this->teamRoleManager->getRole($roleId);

            if($role == null) {
                throw new \Exception("Cannot create form with invalid id");
            }

            $defaults = [
                'name' => $role->label,
                'roleColor' => $role->color,
                'canUse' => (boolean) $role->applicable,
            ];
        }

        $form->addText('label', 'Název role')->setRequired('Název role musí být zadán')
        ->setHtmlAttribute('class', 'form-control')
        ->setDefaultValue($defaults['name']);

        $roleArrays = array(
            'badge-primary' => "Modrá",
            'badge-warning' => "Žlutá",
            'badge-info' => "Tyrkysová",
            'badge-success' => "Zelená",
            'badge-secondary' => "Šedá",
            'badge-danger' => "Červená",
            'bg-gray' => "Černá",
            'bg-purple' => "Fialová",
            'bg-orange' => "Oranžová",
            'bg-pink' => "Růžová",
            'bg-lime' => "Limetková",
        );

        $form->addSelect('roleColor', 'Barva štítku', $roleArrays)->setRequired('Barva štítku musí být vybrána')
        ->setHtmlAttribute('class', 'form-control')
        ->setDefaultValue($defaults['roleColor']);

        $form->addCheckbox('canUse', 'Použitelné')->setDefaultValue('checked')
        ->setHtmlAttribute('class', 'form-check-input')
        ->setDefaultValue($defaults['canUse']);

        if($roleId != 0) {
            $form->addHidden('roleId', $roleId);
        }

        $form->addSubmit('send');

        if($roleId != 0) {
            $form->onSuccess[] = [$this, 'editTeamRole'];
        } else {
            $form->onSuccess[] = [$this, 'addTeamRole'];
        }

        return $form;
    }

    /**
     * Function that is triggered by a successful form submission
     * @param Form $form
     * @param ArrayHash $values
     */
    public function addTeamRole(Form $form, ArrayHash $values) {
        $this->teamRoleManager->createRole($values['label'], $values['canUse'], $values['roleColor']);
    }

    /**
     * Function that is triggered by a successful form submission
     * @param Form $form
     * @param ArrayHash $values
     */
    public function editTeamRole(Form $form, ArrayHash $values) {
        $this->teamRoleManager->editRole($values['roleId'], $values['label'], $values['canUse'], $values['roleColor']);
    }


}